import { isAfter, isBefore, isSameDay, isSameMonth } from 'date-fns';
function isOutside(day, currentMonth) {
    return !isSameMonth(day, currentMonth);
}
/** Props that will make the day button interactive. */
var interactiveProps = [
    'onSelect',
    'onDayClick',
    'onDayMouseEnter',
    'onDayTouchStart'
];
function isInteractive(day, currentMonth, props) {
    var toDate = props.toDate, fromDate = props.fromDate, originalProps = props.originalProps;
    var outside = isOutside(day, currentMonth);
    if (props.mode !== 'uncontrolled' && !outside) {
        return true;
    }
    // If some of the _original props_ is an interactive prop
    if (interactiveProps.every(function (name) { return !(name in originalProps); })) {
        return false;
    }
    var isAfterToDate = toDate && isAfter(day, toDate);
    var isBeforeFromDate = fromDate && isBefore(day, fromDate);
    var interactive = !isAfterToDate && !isBeforeFromDate && !outside;
    return interactive;
}
function isToday(day, _, props) {
    return isSameDay(day, props.today);
}
export var defaultModifiers = {
    interactive: isInteractive,
    outside: isOutside,
    today: isToday
};
//# sourceMappingURL=defaultModifiers.js.map